<?php

namespace Nwb\S3;

use Aws\S3\S3Client;

class S3
{
    private $configS3;
    protected $s3RootFolder = '';
    const TAG_CONTENT_TYPE_KEY = 'ContentType';

    public function __construct(?ConfigS3 $configS3 = null)
    {
        $this->configS3 = ($configS3 instanceof ConfigS3) ? $configS3 : new ConfigS3();
    }

    public function getConfigS3(): ConfigS3
    {
        return $this->configS3;
    }

    /**
     * Envoi un fichier vers un stockage d'objets S3.
     * Si l'objet existe déjà il sera écrasé sans avertissement.
     * @param string $filePath Le chemin du fichier à charger.
     * @param string $destinationPath Le chemin de destination du fichier sur Amazon S3.
     * @param bool $isPublic Si true, le fichier sera accessible publiquement, sinon il sera privé. Valeur par défaut : false.
     * @return \Aws\Result Le résultat de l'opération de chargement.
     */
    public function upload(string $filePath, string $destinationPath, bool $isPublic = false): \Aws\Result
    {
        $resource = fopen($filePath, 'r');
        if ($resource === false) {
            throw new \RuntimeException("Impossible d'ouvrir le fichier: $filePath");
        }

        try {
            return $this->uploadResource($resource, $destinationPath, $isPublic);
        } finally {
            fclose($resource);
        }
    }

    /**
     * Déplace un fichier vers un stockage d'objets S3 et supprime le fichier source.
     * Appelle la méthode upload pour envoyer le fichier à S3, puis supprime le fichier source localement.
     * @param string $filePath Le chemin du fichier à déplacer.
     * @param string $destinationPath Le chemin de destination du fichier sur Amazon S3.
     * @param bool $isPublic Si true, le fichier sera accessible publiquement, sinon il sera privé. Valeur par défaut : false.
     * @return \Aws\Result Le résultat de l'opération de déplacement.
     * @throws \Exception Si la suppression du fichier source échoue.
     */
    public function moveToS3(string $filePath, string $destinationPath, bool $isPublic = false): \Aws\Result
    {
        // Appel de la méthode upload pour envoyer le fichier à S3
        $result = $this->upload($filePath, $destinationPath, $isPublic);

        // Suppression du fichier source localement
        if (!unlink($filePath)) {
            throw new \Exception("Impossible de supprimer le fichier source: $filePath");
        }

        return $result;
    }

    /**
     * Affiche le contenu d'un fichier S3 dans le navigateur.
     *
     * @param string $nomFichierS3 Le nom du fichier S3 à afficher.
     * @param bool $envoyerContentType Indique si l'en-tête 'Content-Type' doit être envoyé. Par défaut à true.
     * @return void
     */
    public function show(string $nomFichierS3, bool $envoyerContentType = true): void
    {
        $resource = $this->downloadResource($nomFichierS3);

        // Déterminez le Content-Type
        $contentType = $this->determineContentType($nomFichierS3, $resource);

        if ($envoyerContentType && $contentType) {
            header("Content-Type: $contentType");
        }

        // Affiche le contenu de la ressource
        fpassthru($resource);

        // Fermez la ressource
        fclose($resource);
    }

    /**
     * Télécharge un fichier depuis Amazon S3 et l'enregistre localement.
     *
     * @param string $nomFichierS3 Le nom du fichier à télécharger sur S3.
     * @param string $destinationPath Le chemin de destination où le fichier sera enregistré localement.
     * @return void
     */
    public function download(string $nomFichierS3, string $destinationPath): void
    {
        $nomFichierS3 = $this->normalizePath($nomFichierS3);
        $s3 = new S3Client($this->configS3->getConfig());
        $result = $s3->getObject([
            'Bucket' => $this->configS3->getBucket(),
            'Key' => $nomFichierS3,
        ]);
        file_put_contents($destinationPath, $result['Body']);
    }

    /**
     * Envoi une ressource vers un stockage d'objets S3.
     * Si l'objet existe déjà il sera écrasé sans avertissement.
     * @param resource $resource La ressource à charger.
     * @param string $destinationPath Le chemin de destination de la ressource sur Amazon S3.
     * @param bool $isPublic Si true, le fichier sera accessible publiquement, sinon il sera privé. Valeur par défaut : false.
     * @return \Aws\Result Le résultat de l'opération de chargement.
     */
    public function uploadResource($resource, string $destinationPath, bool $isPublic = false): \Aws\Result
    {
        if (!is_resource($resource)) {
            throw new \InvalidArgumentException("uploadResource() The provided resource is invalid.");
        }

        $data = stream_get_contents($resource);

        $tags = [];

        if ($this->configS3->getEncryptionKey()) {
            $data = $this->encryptData($data);

            // Si crypté, stockez le Content-Type dans les métadonnées ou les tags
            $contentType = mime_content_type($resource);

            $tags = [self::TAG_CONTENT_TYPE_KEY => $contentType];
        }

        $acl = $isPublic ? 'public-read' : 'private';
        $destinationPath = $this->normalizePath($destinationPath);
        $s3 = new S3Client($this->configS3->getConfig());
        $result = $s3->putObject([
            'Bucket' => $this->configS3->getBucket(),
            'Key' => $destinationPath,
            'Body' => $data,
            'ACL' => $acl
        ]);

        if (!empty($tags)) {
            $s3->putObjectTagging([
                'Bucket' => $this->configS3->getBucket(),
                'Key' => $destinationPath,
                'Tagging' => [
                    'TagSet' => array_map(function ($key, $value) {
                        return ['Key' => $key, 'Value' => $value];
                    }, array_keys($tags), $tags)
                ],
            ]);
        }

        return $result;
    }

    /**
     * Supprime plusieurs fichiers du bucket S3.
     *
     * @param array $fileNames Tableau de noms de fichiers à supprimer.
     * @throws S3Exception Si la suppression échoue.
     */
    private function deleteMultiple(array $fileNames): void
    {
        $objects = array_map(function ($fileName) {
            return ['Key' => $this->normalizePath($fileName)];
        }, $fileNames);

        $s3 = new S3Client($this->configS3->getConfig());
        try {
            $result = $s3->deleteObjects([
                'Bucket'  => $this->configS3->getBucket(),
                'Delete' => ['Objects' => $objects],
            ]);
        } catch (\Aws\Exception\AwsException $e) {
            throw new S3Exception("Erreur lors de la suppression des fichiers.", 0, $e);
        }
    }

    /**
     * Supprime un ou plusieurs fichiers du bucket S3.
     *
     * @param mixed $nomFichierS3 Le nom du fichier (string) ou un tableau de noms de fichiers à supprimer.
     * @throws S3Exception Si la suppression échoue.
     */
    public function delete($nomFichierS3): void
    {
        $fileNames = is_array($nomFichierS3) ? $nomFichierS3 : [$nomFichierS3];
        $this->deleteMultiple($fileNames);
    }

    /**
     * Normalisation du chemin d'accès S3
     * Supprime le slash au début du chemin si présent et ajoute le dossier racine (si défini)
     * @param string $chemin
     * @return string
     */
    private function normalizePath(string $chemin): string
    {
        if (substr($chemin, 0, 1) === '/') {
            $chemin = substr($chemin, 1);
        }
        return $this->s3RootFolder . $chemin;
    }

    /**
     * Renvoie vrai si le fichier existe sur S3
     * @param string $nomFichierS3
     * @return bool
     */
    public function fileExists(string $nomFichierS3): bool
    {
        $nomFichierS3 = $this->normalizePath($nomFichierS3);
        $s3 = new S3Client($this->configS3->getConfig());
        return $s3->doesObjectExist($this->configS3->getBucket(), $nomFichierS3);
    }

    /**
     * Liste les fichiers dans un chemin spécifié dans le bucket S3.
     * Peut lister récursivement si spécifié.
     *
     * @param string $path Le chemin à l'intérieur du bucket où lister les fichiers.
     * @param bool $recursive Indique si la liste doit être récursive.
     * @return array Un tableau contenant les emplacements relatif des fichiers (relatif à $path).
     */
    public function listFiles(string $path = '', bool $recursive = false): array
    {
        $path = $this->normalizeFolderPath($path);
        $s3 = new S3Client($this->configS3->getConfig());

        $fileList = [];
        $marker = null;

        do {
            $options = [
                'Bucket' => $this->configS3->getBucket(),
                'Prefix' => $path
            ];

            if (!$recursive) {
                $options['Delimiter'] = '/';
            }

            if ($marker) {
                $options['Marker'] = $marker;
            }

            $objects = $s3->listObjects($options);

            if (isset($objects['Contents']) && is_array($objects['Contents'])) {
                foreach ($objects['Contents'] as $object) {
                    $relativePath = substr($object['Key'], strlen($path));
                    $relativePath = ltrim($relativePath, '/');
                    if ($relativePath !== '') {
                        $fileList[] = $relativePath;
                    }
                }
            }

            // Ajoute aussi les pseudo-dossiers (CommonPrefixes) à la liste
            if (isset($objects['CommonPrefixes']) && is_array($objects['CommonPrefixes'])) {
                foreach ($objects['CommonPrefixes'] as $prefix) {
                    $relativePath = substr($prefix['Prefix'], strlen($path));
                    $relativePath = rtrim(ltrim($relativePath, '/'), '/');
                    if ($relativePath !== '') {
                        $fileList[] = $relativePath . '/'; // Ajoute un slash pour indiquer un dossier
                    }
                }
            }

            $marker = $objects['NextMarker'] ?? null;
        } while ($objects['IsTruncated']);

        return $fileList;
    }

    /**
     * Télécharge un fichier depuis Amazon S3 et retourne la ressource de flux.
     *
     * @param string $nomFichierS3 Le nom du fichier à télécharger sur S3.
     * @return resource La ressource de flux du fichier téléchargé.
     * @throws S3Exception Si la récupération échoue.
     */
    public function downloadResource(string $nomFichierS3)
    {
        $nomFichierS3 = $this->normalizePath($nomFichierS3);
        $s3 = new S3Client($this->configS3->getConfig());

        try {
            $result = $s3->getObject([
                'Bucket' => $this->configS3->getBucket(),
                'Key' => $nomFichierS3,
            ]);
        } catch (\Aws\Exception\AwsException $e) {
            throw new S3Exception("Erreur lors du téléchargement du fichier depuis S3.", 0, $e);
        }

        $data = (string)$result['Body'];
        if ($this->configS3->getEncryptionKey()) {
            $data = $this->decryptData($data);
        }

        $stream = fopen('php://temp', 'r+');
        fwrite($stream, $data);
        rewind($stream);

        return $stream;
    }


    private function encryptData($data)
    {
        $key = $this->configS3->getEncryptionKey();
        if (!$key) {
            throw new \RuntimeException("Aucune clé de chiffrement définie.");
        }
        $iv = openssl_random_pseudo_bytes(openssl_cipher_iv_length('aes-256-cbc'));
        $encryptedData = openssl_encrypt($data, 'aes-256-cbc', $key, OPENSSL_RAW_DATA, $iv);

        // Concatène l'IV et les données chiffrées (sans Base64)
        return $iv . $encryptedData;
    }

    private function decryptData($data)
    {
        $key = $this->configS3->getEncryptionKey();
        if (!$key) {
            throw new \RuntimeException("Aucune clé de chiffrement définie.");
        }
        $ivSize = openssl_cipher_iv_length('aes-256-cbc');
        $iv = substr($data, 0, $ivSize);
        $encryptedData = substr($data, $ivSize);

        return openssl_decrypt($encryptedData, 'aes-256-cbc', $key, OPENSSL_RAW_DATA, $iv);
    }

    private function determineContentType(string $nomFichierS3, $resource)
    {
        if ($this->configS3->getEncryptionKey()) {
            $metadata = $this->getTags($nomFichierS3);
            return $metadata[self::TAG_CONTENT_TYPE_KEY] ?? null;
        }

        return mime_content_type($resource);
    }

    /**
     * Récupère les tags d'un fichier dans le bucket S3.
     *
     * @param string $nomFichierS3 Le nom du fichier dont on veut récupérer les tags.
     * @return array Les tags du fichier.
     * @throws S3Exception Si la récupération échoue.
     */
    public function getTags(string $nomFichierS3): array
    {
        $nomFichierS3 = $this->normalizePath($nomFichierS3);
        $s3 = new S3Client($this->configS3->getConfig());

        try {
            // Récupération des tags
            $tagsResult = $s3->getObjectTagging([
                'Bucket' => $this->configS3->getBucket(),
                'Key' => $nomFichierS3,
            ]);
            $tags = $tagsResult->get('TagSet');

            // Transformer les tags en un tableau associatif plus simple
            $formattedTags = [];
            foreach ($tags as $tag) {
                $formattedTags[$tag['Key']] = $tag['Value'];
            }

            return $formattedTags;
        } catch (\Aws\Exception\AwsException $e) {
            throw new S3Exception("Erreur lors de la récupération des tags du fichier depuis S3.", 0, $e);
        }
    }

    /**
     * Crée un pseudo-dossier dans S3.
     *
     * @param string $folderPath Chemin du dossier à créer.
     * @throws S3Exception Si la création échoue.
     */
    public function createFolder(string $folderPath): void
    {
        // Normalise et assure que le chemin se termine par un slash
        $folderPath = $this->normalizeFolderPath($folderPath);

        try {
            $s3 = new S3Client($this->configS3->getConfig());

            // Tente de créer une clé vide pour simuler un dossier
            $s3->putObject([
                'Bucket' => $this->configS3->getBucket(),
                'Key' => $folderPath,
                'Body' => '',
            ]);
        } catch (\Aws\Exception\AwsException $e) {
            // Lance une exception générale en cas d'erreur avec AWS
            throw new S3Exception("Échec de la création du dossier.", 0, $e);
        }
    }

    private function normalizeFolderPath(string $folderPath): string
    {
        if ($folderPath === '/' || $folderPath == '') {
            return '';
        }

        $folderPath = $this->normalizePath($folderPath);
        if (substr($folderPath, -1) !== '/') {
            $folderPath .= '/';
        }
        return $folderPath;
    }

    /**
     * Supprime un dossier et son contenu du bucket S3 de manière récursive.
     *
     * @param string $folderPath Le chemin du dossier à supprimer.
     * @throws S3Exception Si la suppression échoue.
     */
    public function deleteFolder(string $folderPath): void
    {
        $folderPath = $this->normalizePath($folderPath);
        // Assurez-vous que le chemin du dossier se termine par un slash
        if (substr($folderPath, -1) !== '/') {
            $folderPath .= '/';
        }

        // Récupère tous les fichiers et sous-dossiers
        $items = $this->listFiles($folderPath, true);

        // Prépare les chemins complets pour la suppression
        $itemsToDelete = array_map(function ($item) use ($folderPath) {
            return $folderPath . $item;
        }, $items);

        // Si le dossier contient des éléments, les supprimer
        if (!empty($itemsToDelete)) {
            try {
                $this->delete($itemsToDelete);
            } catch (\Exception $e) {
                throw new S3Exception("Erreur lors de la suppression du dossier", 0, $e);
            }
        }

        // Tente de supprimer le dossier lui-même en fin de processus
        // Utile si le dossier est vide ou pour s'assurer que le "pseudo-dossier" est supprimé
        try {
            $this->delete($folderPath);
        } catch (\Exception $e) {
            // Gestion des erreurs si le dossier n'existe pas comme un objet distinct
            // Ceci est optionnel et dépend de votre gestion d'erreur spécifique
            // Ne lance pas d'exception si le dossier n'existe pas
        }
    }

    /**
     * Génère une URL présignée pour un fichier S3.
     *
     * @param string $nomFichierS3 Le nom du fichier S3 pour lequel générer l'URL présignée.
     * @param string $expiration La durée de validité de l'URL présignée (par exemple, '+10 minutes').
     * @param bool $inline Si true, le fichier sera affiché directement dans le navigateur s'il est pris en charge, sinon il sera téléchargé.
     * @param string|null $setContentType Le type MIME du fichier pour le Content-Type de la réponse, ou null pour ne pas le spécifier.
     * @return string L'URL présignée.
     * @throws S3Exception Si la génération de l'URL échoue.
     */
    public function getPresignedUrl(string $nomFichierS3, string $expiration = '+10 minutes', bool $inline = false, ?string $setContentType = null): string
    {
        $nomFichierS3 = $this->normalizePath($nomFichierS3);
        $s3 = new S3Client($this->configS3->getConfig());

        $contentDisposition = $inline ? 'inline' : 'attachment'; // Définir le type de Content-Disposition
        $cmd = $s3->getCommand('GetObject', [
            'Bucket' => $this->configS3->getBucket(),
            'Key' => $nomFichierS3,
            'ResponseContentDisposition' => $contentDisposition
        ]);

        if ($setContentType !== null) {
            $cmd['ResponseContentType'] = $setContentType;
        }

        $request = $s3->createPresignedRequest($cmd, $expiration);

        return (string)$request->getUri();
    }
    /**
     * Envoi un fichier vers un stockage d'objets S3 avec une durée de vie.
     * Si l'objet existe déjà, il sera écrasé sans avertissement.
     *
     * @param string $filePath Le chemin du fichier à charger.
     * @param string $destinationPath Le chemin de destination du fichier sur Amazon S3.
     * @param int $expirationDays La durée de vie du fichier en jours.
     * @param bool $isPublic Si true, le fichier sera accessible publiquement, sinon il sera privé. Valeur par défaut : false.
     * @return \Aws\Result Le résultat de l'opération de chargement.
     */
    public function uploadWithExpiration(string $filePath, string $destinationPath, int $expirationDays, bool $isPublic = false): \Aws\Result
    {
        $resource = fopen($filePath, 'r');
        if ($resource === false) {
            throw new \RuntimeException("Impossible d'ouvrir le fichier: $filePath");
        }

        try {
            // Envoyer le fichier
            $result = $this->uploadResource($resource, $destinationPath, $isPublic);

            // Ajouter le tag de date d'expiration
            $this->setExpirationTag($destinationPath, $expirationDays);

            return $result;
        } finally {
            fclose($resource);
        }
    }

    /**
     * Ajoute un tag de durée de vie à un objet S3.
     *
     * @param string $destinationPath Le chemin de l'objet sur Amazon S3.
     * @param int $expirationDays La durée de vie du fichier en jours.
     */
    private function setExpirationTag(string $destinationPath, int $expirationDays): void
    {
        $destinationPath = $this->normalizePath($destinationPath);
        $s3 = new S3Client($this->configS3->getConfig());

        $tags = [
            [
                'Key' => 'expiration-in-day',
                'Value' => (string)$expirationDays,
            ],
        ];

        $s3->putObjectTagging([
            'Bucket' => $this->configS3->getBucket(),
            'Key' => $destinationPath,
            'Tagging' => ['TagSet' => $tags],
        ]);

        // Ajouter la configuration du cycle de vie pour gérer l'expiration
        $this->addLifecycleRule($expirationDays);
    }

    /**
     * Ajoute une règle de cycle de vie pour supprimer les objets avec un tag d'expiration.
     *
     * @param int $expirationDays La durée de vie du fichier en jours.
     */
    private function addLifecycleRule(int $expirationDays): void
    {
        $ruleId = "expiration-rule-{$expirationDays}-days";
        $s3 = new S3Client($this->configS3->getConfig());
        $bucket = $this->configS3->getBucket();

        // Récupérer la configuration actuelle du cycle de vie
        try {
            $result = $s3->getBucketLifecycleConfiguration([
                'Bucket' => $bucket,
            ]);
            $rules = $result['Rules'];
        } catch (\Aws\Exception\AwsException $e) {
            if ($e->getAwsErrorCode() === 'NoSuchLifecycleConfiguration') {
                $rules = [];
            } else {
                throw $e;
            }
        }

        // Vérifier si la règle existe déjà
        foreach ($rules as $rule) {
            if ($rule['ID'] === $ruleId) {
                return;
            }
        }

        // Ajouter la nouvelle règle
        $rules[] = [
            'ID' => $ruleId,
            'Status' => 'Enabled',
            'Filter' => [
                'Tag' => [
                    'Key' => 'expiration-in-day',
                    'Value' => (string)$expirationDays,
                ],
            ],
            'Expiration' => [
                'Days' => $expirationDays,
            ],
        ];

        // Mettre à jour la configuration du cycle de vie
        $s3->putBucketLifecycleConfiguration([
            'Bucket' => $bucket,
            'LifecycleConfiguration' => [
                'Rules' => $rules,
            ],
        ]);
    }
}
